/*
 * MinHttpdThread.java
 *
 * Creado el 14 de diciembre de 2007, 10:24
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.mhpsc.minhttpd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author avelino
 */
public class HttpDispatcher implements Runnable {
    protected Socket socket;
    protected HttpProcessor processor;
    protected InputStream request;
    protected OutputStream response;
    private static final int MAX_READED_DATA = 1024;
    
    public HttpDispatcher(Socket socket, HttpProcessor processor) {
        this.socket = socket;
        this.processor = processor;
        try {
            this.request = this.socket.getInputStream();
            this.response = this.socket.getOutputStream();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static String getTranslatedValue(String v) {
        String aux = v.replace('+', ' ');
        String ret = "";
        for (int i = 0; i < aux.length(); i++) {
            char c = aux.charAt(i);
            if (c == '%') {
                byte isoValue = Byte.parseByte(aux.substring(i + 1, i + 3), 16);
                try {
                    ret += new String(new byte[]{isoValue}, "ISO-8859-1");
                } catch (UnsupportedEncodingException ex) {
                }
                i += 2;
            }
            else
                ret += c;
        }
        return ret;
    }
    
    public void run() {
        System.out.println("starting run");
        try {
            byte[] readedData = new byte[MAX_READED_DATA];
            int readed = this.request.read(readedData, 0, MAX_READED_DATA);
            String req = new String(readedData, 0, readed, "ISO-8859-1");
            //System.out.println("request: '" + req + "' (" + readed + " bytes)");
            Pattern pattern = Pattern.compile("([a-zA-Z]+)[ ]+([^ ]+)[ ]+([^ ]+)\r\n(([^\\:\r\n]+)\\:[ ]+([^\r\n]+)\r\n)*");
            StringBuffer stringBuffer = new StringBuffer(req);
            Matcher matcher = pattern.matcher(stringBuffer);
            if (matcher.find()) {
                String method = matcher.group(1);
                String resource = matcher.group(2);
                String protocol = matcher.group(3);
                Hashtable<String, String> header = new Hashtable<String, String>();
                Hashtable<String, String> variables = new Hashtable<String, String>();
                System.out.println("method='" + method + "'    resource='" + resource + "'");
                String[] sp = req.split("\r\n");
                for (int i = 1; i < sp.length; i++) {
                    String[] sp1 = sp[i].split("\\: ");
                    if (sp1.length != 2)
                        break;
                    String headerVariable = sp1[0];
                    String headerValue = sp1[1];
                    System.out.println("        headerVariable='" + headerVariable + "'      headerValue='" + headerValue + "'");
                    header.put(headerVariable, headerValue);
                }
                String variableString = "";
                byte[] postData = new byte[]{};
                if (method.compareToIgnoreCase("POST") == 0) {
                    String contentLength = header.get("Content-Length");
                    byte[] auxPostData = stringBuffer.substring(matcher.end() + 2).getBytes("ISO-8859-1");
                    int dataLength = (contentLength == null) ? auxPostData.length : Integer.parseInt(contentLength);
                    postData = new byte[dataLength];
                    System.arraycopy(auxPostData, 0, postData, 0, dataLength);
                    System.out.println("POST DATA " + dataLength + " byte(s)");
                    String contentType = header.get("Content-Type");
                    if (contentType != null) {
                        if (contentType.compareToIgnoreCase("application/x-www-form-urlencoded") == 0)
                            variableString = new String(postData, "ISO-8859-1");
                    }
                }
                else if (method.compareToIgnoreCase("GET") == 0) {
                    // TODO
                    pattern = Pattern.compile("[^\\?]+\\?([^ ]+)");
                    stringBuffer = new StringBuffer(resource);
                    matcher = pattern.matcher(stringBuffer);
                    if (matcher.matches())
                        variableString = matcher.group(1);
                }
                else
                    throw new IOException("Invalid method '" + method + "'");
                System.out.println("VARIABLE STRING: '" + variableString + "'");
                parseVariableString(variableString, variables, "&");
                HttpRequest httpRequest = new HttpRequest(method, resource, protocol, header, variables, postData);
                HttpResponse httpResponse = new HttpResponse(httpRequest);
                if (this.processor == null) {
                    // put a default response
                    String defaultMessage = "No HttpProcessor defined";
                    String res = protocol + " 200\r\nContent-Type: text/plain\r\nContent-Length: " + defaultMessage.getBytes("ISO-8859-1").length + "\r\n\r\n" + defaultMessage;
                    this.response.write(res.getBytes("ISO-8859-1"));
                }
                else {
                    this.processor.processHttpRequest(httpRequest, httpResponse);
                    String res = protocol + " " + httpResponse.status + "\r\n" + httpResponse.getHeader();
                    this.response.write(res.getBytes("ISO-8859-1"));
                    if (httpResponse.outputStream.size() > 0) {
                        byte[] responseData = httpResponse.outputStream.toByteArray();
                        res = "Content-Length: " + responseData.length + "\r\n\r\n";
                        this.response.write(res.getBytes("ISO-8859-1"));
                        this.response.write(httpResponse.outputStream.toByteArray());
                    }
                    else {
                        res = "\r\n";
                        this.response.write(res.getBytes("ISO-8859-1"));
                    }
                }
                this.response.flush();
                this.socket.close();
            }
            else {
                System.out.println("does not match pattern:");
                System.out.println("-------------------");
                System.out.println(req);
                System.out.println("-------------------");
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("ending run");
    }
    
    public static void parseVariableString(String variableString, Hashtable<String,String> variables, String separator) {
        String[] trunks = variableString.split("&");
        Pattern pattern = Pattern.compile("([^=]+)=(.*)");
        //System.out.println("parseVariableString:                               " + trunks.length + " trunk(s)");
        for (String trunk : trunks) {
            Matcher matcher = pattern.matcher(new StringBuffer(trunk));
            if (matcher.find()) {
                String variableName = matcher.group(1).trim();
                String variableValue = getTranslatedValue(matcher.group(2).trim());
                //System.out.println("parseVariableString:                               name='" + variableName + "' value='" + variableValue + "'");
                variables.put(variableName, variableValue);
            }
        }
    }
    
}
