/*
 * HttpResponse.java
 *
 * Creado el 17 de diciembre de 2007, 9:40
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.mhpsc.minhttpd;

import java.io.ByteArrayOutputStream;
import java.util.Hashtable;

/**
 *
 * @author avelino
 */
public class HttpResponse {
    private HttpRequest request;
    public int status;
    public Hashtable<String,String> header;
    public ByteArrayOutputStream outputStream;
    // informational status codes
    public static final int STATUS_CONTINUE = 100;
    public static final int STATUS_SWITCHING_PROTOCOLS = 101;
    public static final int STATUS_PROCESSING = 102;
    // success status codes
    public static final int STATUS_OK = 200;
    public static final int STATUS_CREATED = 201;
    public static final int STATUS_ACCEPTED = 202;
    public static final int STATUS_NON_AUTHORITATIVE_INFO = 203;
    public static final int STATUS_NO_CONTENT = 204;
    public static final int STATUS_RESET_CONTENT = 205;
    public static final int STATUS_PARTIAL_CONTENT = 206;
    public static final int STATUS_MULTI_STATUS = 207;
    // redirection status codes
    public static final int STATUS_MULTIPLE_CHOICES = 300;
    public static final int STATUS_MOVED_PERMANENTLY = 301;
    public static final int STATUS_FOUND = 302;
    public static final int STATUS_SEE_OTHER = 303;
    public static final int STATUS_NOT_MODIFIED = 304;
    public static final int STATUS_USE_PROXY = 305;
    public static final int STATUS_SWITCH_PROXY = 306;
    public static final int STATUS_TEMPORARY_REDIRECT = 307;
    // client error status codes
    public static final int STATUS_BAD_REQUEST = 400;
    public static final int STATUS_UNAUTHORIZED = 401;
    public static final int STATUS_PAYMENT_REQUIRED = 402;
    public static final int STATUS_FORBIDDEN = 403;
    public static final int STATUS_NOT_FOUND = 404;
    public static final int STATUS_METHOD_NOT_ALLOWED = 405;
    public static final int STATUS_NOT_ACCEPTABLE = 406;
    public static final int STATUS_PROXY_AUTHENTICATION_REQUIRED = 407;
    public static final int STATUS_REQUEST_TIMEOUT = 408;
    public static final int STATUS_CONFLICT = 409;
    public static final int STATUS_GONE = 410;
    public static final int STATUS_LENGTH_REQUIRED = 411;
    public static final int STATUS_PRECONDITION_FAILED = 412;
    public static final int STATUS_REQUEST_ENTITY_TOO_LARGE = 413;
    public static final int STATUS_REQUEST_URI_TOO_LONG = 414;
    public static final int STATUS_UNSUPPORTED_MEDIA_TYPE = 415;
    public static final int STATUS_REQUESTED_RANGE_NOT_SATISFIABLE = 416;
    public static final int STATUS_EXPECTATION_FAILED = 417;
    public static final int STATUS_UNPROCESSABLE_ENTITY = 422;
    public static final int STATUS_LOCKED = 423;
    public static final int STATUS_FAILED_DEPENDENCY = 424;
    public static final int STATUS_UNORDERED_COLLECTION = 425;
    public static final int STATUS_UPGRADE_REQUIRED = 426;
    public static final int STATUS_RETRY_WITH = 449;
    // server error status codes
    public static final int STATUS_INTERNAL_SERVER_ERROR = 500;
    public static final int STATUS_NOT_IMPLEMENTED = 501;
    public static final int STATUS_BAD_GATEWAY = 502;
    public static final int STATUS_SERVICE_UNAVAILABLE = 503;
    public static final int STATUS_GATEWAY_TIMEOUT = 504;
    public static final int STATUS_HTTP_VERSION_NOT_SUPPORTED = 505;
    public static final int STATUS_VARIANT_ALSO_NEGOTIATES = 506;
    public static final int STATUS_INSUFFICIENT_STORAGE = 507;
    public static final int STATUS_BANDWIDTH_LIMIT_EXCEEDED = 509;
    public static final int STATUS_NOT_EXTENDED = 510;
    
    public HttpResponse(HttpRequest request) {
        this.request = request;
        this.status = 200;
        this.header = new Hashtable<String,String>();
        this.outputStream = new ByteArrayOutputStream();
    }

    public void addCookiePair(String name, String value) {
        this.request.cookie.put(name, value);
    }
    
    public String getHeader() {
        String ret = "";
        if (!this.request.cookie.isEmpty()) {
            ret += "Set-Cookie:";
            for (String name : this.request.cookie.keySet()) {
                String value = this.request.cookie.get(name);
                ret += " " + name + "=" + value + ";";
            }
            ret += "\r\n";
        }
        for (String headerVariable : this.header.keySet()) {
            String headerValue = this.header.get(headerVariable);
            ret += headerVariable + ": " + headerValue + "\r\n";
        }
        //System.out.println("HttpResponse.getHeader(): <<<" + ret + ">>>");
        return ret;
    }
}
