/*
 * NIOTest.java
 *
 * Creado el 18 de diciembre de 2007, 11:28
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.mhpsc.minhttpd;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author avelino
 */
public class NIOTest {
    
    public static void writeData(SocketChannel socket, byte[] data) throws IOException {
        ByteBuffer byteBuffer = ByteBuffer.wrap(data);
        socket.write(byteBuffer);
    }

    public static int readData(SelectionKey key, byte[] data) throws IOException {
        SocketChannel socket = (SocketChannel) key.channel();
        ByteBuffer byteBuffer = ByteBuffer.allocate(1024);
        int ret = socket.read(byteBuffer);
        byte[] byteBufferArray = byteBuffer.array();
        if (ret > 0)
            System.arraycopy(byteBufferArray, 0, data, 0, ret);
        return ret;
    }
    
    public static void main(String[] args) {
        try {
            Selector selector = Selector.open();   
            ServerSocketChannel serverSocketChannel = ServerSocketChannel.open();
            serverSocketChannel.configureBlocking(false);
            InetAddress ia = InetAddress.getLocalHost();
            InetSocketAddress isa = new InetSocketAddress(ia, 1799);
            serverSocketChannel.socket().bind(isa);
            SelectionKey selectionKey = serverSocketChannel.register(selector, SelectionKey.OP_ACCEPT);
            while (selectionKey.selector().select() > 0) {
                Set<SelectionKey> readyKeys = selector.selectedKeys();
                Iterator<SelectionKey> it = readyKeys.iterator();
                while (it.hasNext()) {
                    SelectionKey key = it.next();
                    it.remove();
                    SocketChannel socket = null;
                    if (key.isAcceptable()) {
                        System.out.println("acceptable key");
                        ServerSocketChannel ssc = (ServerSocketChannel) key.channel();
                        socket = (SocketChannel) ssc.accept();
                        socket.configureBlocking(false);
                        SelectionKey another = socket.register(selector, SelectionKey.OP_READ | SelectionKey.OP_WRITE);
                    }
                    if (key.isReadable()) {
                        byte[] buffer = new byte[1024];
                        int readed = readData(key, buffer);
                        if (readed > 0) {
                            String ret = new String(buffer, 0, readed, "ISO-8859-1");
                            System.out.println("readable key: echoing '" + ret + "'");
                            socket = (SocketChannel) key.channel();
                            writeData(socket, ret.getBytes("ISO-8859-1"));
                        }
                    }
                    if (key.isWritable()) {
                        byte[] buffer = new byte[1024];
                        int readed = readData(key, buffer);
                        if (readed > 0) {
                            String ret = new String(buffer, 0, readed, "ISO-8859-1");
                            System.out.println("writable key: echoing '" + ret + "'");
                            socket = (SocketChannel) key.channel();
                            writeData(socket, ret.getBytes("ISO-8859-1"));
                        }
                    }
                }
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
}
